import React from "react";
import { useParams } from "react-router-dom";
import { VMApi, VMInfo } from "../api/VMApi";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { VirtWebRouteContainer } from "../widgets/VirtWebRouteContainer";
import { IconButton } from "@mui/material";
import { RouterLink } from "../widgets/RouterLink";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import { XMLWidget } from "../widgets/XMLWidget";

export function VMXMLRoute(): React.ReactElement {
  const { uuid } = useParams();

  const [vm, setVM] = React.useState<VMInfo | undefined>();
  const [src, setSrc] = React.useState<string | undefined>();

  const load = async () => {
    setVM(await VMApi.GetSingle(uuid!));
    setSrc(await VMApi.GetSingleXML(uuid!));
  };

  return (
    <AsyncWidget
      loadKey={uuid}
      load={load}
      errMsg="Failed to load VM information!"
      build={() => <XMLRouteInner vm={vm!} src={src!} />}
    />
  );
}

function XMLRouteInner(p: { vm: VMInfo; src: string }): React.ReactElement {
  return (
    <VirtWebRouteContainer
      label={`XML definition of ${p.vm.name}`}
      actions={
        <RouterLink to={p.vm.ViewURL}>
          <IconButton>
            <ArrowBackIcon />
          </IconButton>
        </RouterLink>
      }
    >
      <XMLWidget src={p.src} />
    </VirtWebRouteContainer>
  );
}
